==> mutt-0.48.1-h7.4.tgz: mutt-0.48.1-h7.4/help.c <==
/*
 * Copyright (C) 1996 Michael R. Elkins <me@cs.hmc.edu>
 * 
 *     This program is free software; you can redistribute it and/or modify
 *     it under the terms of the GNU General Public License as published by
 *     the Free Software Foundation; either version 2 of the License, or
 *     (at your option) any later version.
 * 
 *     This program is distributed in the hope that it will be useful,
 *     but WITHOUT ANY WARRANTY; without even the implied warranty of
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *     GNU General Public License for more details.
 * 
 *     You should have received a copy of the GNU General Public License
 *     along with this program; if not, write to the Free Software
 *     Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */ 

#include "mutt.h"
#include "mutt_curses.h"
#include "keymap.h"

static struct binding_t *help_lookupFunction (int op, int menu)
{
  int i;

  switch (menu) {
  case MENU_MAIN:
    for (i=0; Op_main[i].name; i++)
      if (Op_main[i].op == op)
        return(&Op_main[i]);
    break;

  case MENU_SEND:
    for (i=0; Op_send[i].name; i++)
      if (Op_send[i].op == op)
        return(&Op_send[i]);
    break;

  case MENU_PAGER:
    for (i=0; Op_pager[i].name; i++)
      if (Op_pager[i].op == op)
	return(&Op_pager[i]);
    for (i=0; Op_main[i].name; i++)
      if (Op_main[i].op == op)
        return(&Op_main[i]);
    break;

  default:
    for (i=0; Op_generic[i].name; i++) {
      if (Op_generic[i].op == op)
	return(&Op_generic[i]);    
    }
    if (menu == MENU_ATTACH) {
      for (i=0; Op_attach[i].name; i++)
	if (Op_attach[i].op == op)
	  return(&Op_attach[i]);
    }
    else if (menu == MENU_SENDATTACH) {
      for (i=0; Op_sendattach[i].name; i++)
	if (Op_sendattach[i].op == op)
	  return(&Op_sendattach[i]);
    }
  }
  return 0;
}
 
static char *help_keyname (int c)
{
  static char buf[3];

  switch(c) {
  case ' ':
    return "<space>";
  case '\n':
    return "<return>";
  case '\t':
    return "<tab>";
  case KEY_UP:
    return "<up>";
  case KEY_DOWN:
    return "<down>";
  case KEY_LEFT:
    return "<left>";
  case KEY_RIGHT:
    return "<right>";
  case KEY_NPAGE:
    return "<pagedown>";
  case KEY_PPAGE:
    return "<pageup>";
  case 127:
    return "<delete>";
  }

  if (c > 0 && c <= ctrl('Z')) {
    buf[0] = '^';
    buf[1] = c + '@';
    buf[2] = 0;
    return(buf);
  }
  else {
    snprintf(buf, sizeof(buf), "%c", (unsigned char)c);
    return buf;
  }
}

void ci_help_listKeys (int menu)
{
  char t[_POSIX_PATH_MAX];
  char banner[SHORT_STRING];
  char *desc;
  struct binding_t *p;
  struct keymap *k;
  FILE *f;
  int i;

  mutt_mktemp(t);
  if ((f = fopen(t, "w")) == 0) {
    mutt_error("Could not create temp file!");
    return;
  }
  switch(menu) {
  case MENU_PAGER:
    desc="internal-pager";
    break;
  case MENU_MAIN:
    desc="main-menu";
    break;
  case MENU_ATTACH:
    desc="attach-menu";
    break;
  case MENU_SEND:
    desc="send-menu";
    break;
  case MENU_FOLDER:
    desc="folder-menu";
    break;
  case MENU_ALIAS:
    desc="alias-menu";
    break;
  case MENU_SENDATTACH:
    desc="send-attach-menu";
    break;
  case MENU_URL:
    desc="url-menu";
    break;
  default:
    desc="<UNKNOWN MENU>";
  }
  snprintf(banner, sizeof(banner), "Help for %s", desc);
  for (i = 0; i < 128; i++) {
    if ((k = Keymap[i])) {
      while (k) {
        if (k->menu == menu) {
	  p = help_lookupFunction(k->function, menu);
          fprintf(f, "%-8s   ", help_keyname(i));
          if (p)
	    fprintf(f, "%-18s %s\n", p->name, p->description ? p->description : "No description");
	  else
	    fprintf(f, "ERROR: unlisted function.\n");
          break;
        }
        k = k->next;
      }
    }
  }
  fclose(f);
  ci_simple_pager(banner, t, FALSE);
}

/* returns 1 if screen needs to be redrawn */
int ci_help (int menu)
{
  int ch;

  mvaddstr(LINES-1, 0, "Enter key ('?' = list all bindings): ");
  refresh();
  ch = ci_getch();
  if (ch == '?') {
    CLEARLINE(LINES-1);
    ci_help_listKeys(menu);
    return 1;
  }
  else {
    KEYMAP *k;
    
    if (ch < MAX_KEYS)
      k = Keymap[ch];
    else
      k = 0;
    while (k) {
      if (k->menu == menu) {
	struct binding_t *p = help_lookupFunction(k->function, menu);

	if (p) {
	  mutt_error("'%s' is bound to: %s [%s]",
		     help_keyname(ch),
		     p->name,
		     p->description ? p->description : "no description");
	}
	else
	  mutt_error("ERROR: function is not listed!");
	break;
      }
      k = k->next;
    }
    if (!k) mutt_error("Key is not bound to a function.");
    return 0;
  }
  /* not reached */
}

==> mutt-0.56-h8.1.tgz: mutt-0.56-h8.1/help.c <==
/*
 * Copyright (C) 1996 Michael R. Elkins <me@cs.hmc.edu>
 * 
 *     This program is free software; you can redistribute it and/or modify
 *     it under the terms of the GNU General Public License as published by
 *     the Free Software Foundation; either version 2 of the License, or
 *     (at your option) any later version.
 * 
 *     This program is distributed in the hope that it will be useful,
 *     but WITHOUT ANY WARRANTY; without even the implied warranty of
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *     GNU General Public License for more details.
 * 
 *     You should have received a copy of the GNU General Public License
 *     along with this program; if not, write to the Free Software
 *     Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */ 

#include "mutt.h"
#include "mutt_curses.h"
#include "keymap.h"

static struct binding_t *help_lookupFunction (int op, int menu)
{
  int i;

  switch (menu) {
  case MENU_MAIN:
    for (i=0; Op_main[i].name; i++)
      if (Op_main[i].op == op)
        return(&Op_main[i]);
    break;

  case MENU_SEND:
    for (i=0; Op_send[i].name; i++)
      if (Op_send[i].op == op)
        return(&Op_send[i]);
    break;

  case MENU_PAGER:
    for (i=0; Op_pager[i].name; i++)
      if (Op_pager[i].op == op)
	return(&Op_pager[i]);
    for (i=0; Op_main[i].name; i++)
      if (Op_main[i].op == op)
        return(&Op_main[i]);
    break;

  default:
    for (i=0; Op_generic[i].name; i++)
    {
      if (Op_generic[i].op == op)
	return(&Op_generic[i]);    
    }
    if (menu == MENU_ATTACH)
    {
      for (i=0; Op_attach[i].name; i++)
	if (Op_attach[i].op == op)
	  return(&Op_attach[i]);
    }
    else if (menu == MENU_SENDATTACH)
    {
      for (i=0; Op_sendattach[i].name; i++)
	if (Op_sendattach[i].op == op)
	  return(&Op_sendattach[i]);
    }
  }
  return 0;
}
 
static char *help_keyname (int c)
{
  static char buf[3];

  switch (c) {
  case '\033':
    return "ESC";
  case ' ':
    return "SPC";
  case '\n':
  case '\r':
    return "RET";
  case '\t':
    return "TAB";
  case KEY_UP:
    return "Up";
  case KEY_DOWN:
    return "Down";
  case KEY_LEFT:
    return "Left";
  case KEY_RIGHT:
    return "Right";
  case KEY_NPAGE:
    return "PageDn";
  case KEY_PPAGE:
    return "PageUp";
  case KEY_BACKSPACE:
    return "BackSpace";
  case KEY_HOME:
    return "Home";
  case KEY_END:
    return "End";
  case 127:
    return "DEL";
  }

  if (iscntrl (c))
  {
    buf[0] = '^';
    buf[1] = c + '@';
    buf[2] = 0;
    return (buf);
  }
  else
  {
    snprintf (buf, sizeof(buf), "%c", (unsigned char)c);
    return buf;
  }
}

void print_help (int menu, struct keymap_t *map, char *pfx, FILE *fp)
{
  char buf[SHORT_STRING];
  struct binding_t *b;

  while (map)
  {
    if (map->seq)
    {
      snprintf(buf, sizeof(buf), "%s %s", pfx, help_keyname (map->key));
      print_help(menu, map->seq, buf, fp);
    }
    else if (map->op == OP_MACRO)
      fprintf (fp, "%s %s\t%-20s\t%s\n", pfx, help_keyname (map->key), "macro",
	       map->macro);
    else if (map->op != OP_NULL)
    {
      b = help_lookupFunction (map->op, menu);
      fprintf (fp, "%s %s\t%-20s\t%s\n", pfx, help_keyname (map->key), b->name,
	       b->description);
    }
    map = map->next;
  }
}

void ci_help (int menu)
{
  char t[_POSIX_PATH_MAX];
  char banner[SHORT_STRING];
  char *desc;
  struct keymap_t *k;
  FILE *f;

  mutt_mktemp (t);
  if ((f = fopen(t, "w")) == NULL)
  {
    mutt_error("Could not create temp file!");
    return;
  }
  switch(menu) {
  case MENU_PAGER:
    desc="internal-pager";
    break;
  case MENU_MAIN:
    desc="main-menu";
    break;
  case MENU_ATTACH:
    desc="attach-menu";
    break;
  case MENU_SEND:
    desc="send-menu";
    break;
  case MENU_FOLDER:
    desc="folder-menu";
    break;
  case MENU_ALIAS:
    desc="alias-menu";
    break;
  case MENU_SENDATTACH:
    desc="send-attach-menu";
    break;
  case MENU_URL:
    desc="url-menu";
    break;
  case MENU_POST:
    desc="postponed-menu";
    break;
  default:
    desc="<UNKNOWN MENU>";
  }
  snprintf (banner, sizeof (banner), "Help for %s", desc);
  k = Keymaps[menu];
  print_help (menu, Keymaps[menu], "", f);
  fclose (f);
  
  mutt_do_pager (banner, t);
}
